from ultipa.configuration.RequestConfig import RequestConfig
from ultipa.operate.base_extra import BaseExtra
from ultipa.structs import DBType
from ultipa.structs.Property import Property
from ultipa.types import ULTIPA, ULTIPA_RESPONSE
from ultipa.utils import UQLMAKER, CommandList
from ultipa.utils.ResposeFormat import ResponseKeyFormat
from ultipa.utils.propertyUtils import getPropertyTypesDesc
from typing import Tuple, List

BOOL_KEYS = ["index", "lte"]
REPLACE_KEYS = {
    "name": "propertyName",
    "type": "propertyType",
}


class PropertyExtra(BaseExtra):
    '''
    Processing class that defines settings for property related operations.
    '''

    # def listProperty(self, requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.ResponseListProperty:
    # 	'''
    # 	List all properties.

    # 	Args:
    # 		requestConfig: An object of RequestConfig class

    # 	Returns:
    # 		ResponseListProperty

    # 	'''
    # 	return self.showProperty(requestConfig)

    def showProperty(self, dbType: None, schemaName: str = None,
                     requestConfig: RequestConfig = RequestConfig()) -> List[Property]:
        '''
        Show all Node or Edge Schema properties.

        Args:
            dbType: The DBType of data (DBNODE or DBEDGE)

            schema: The name of schema

            requestConfig: An object of RequestConfig class

        Returns:
            ResponseListProperty

        '''
        if dbType != None:
            if dbType == DBType.DBNODE:
                    command = CommandList.showNodeProperty
            elif dbType == DBType.DBEDGE:
                    command = CommandList.showEdgeProperty
            else:
                raise TypeError("dbType must be an instance of DBType.DBNODE or DBType.DBEDGE")
            if schemaName:
                commandp = ['@' + f"`{schemaName}`"]
            else:
                commandp = ''
        else:
            command = CommandList.showProperty
            commandp = ''

        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        uqlMaker.setCommandParams(commandP=commandp)
        # res = self.UqlListSimple(uqlMaker=uqlMaker,
        # 						 responseKeyFormat=ResponseKeyFormat(boolKeys=BOOL_KEYS),
        # 						 isSingleOne=False)
        res = self.uqlSingle(uqlMaker=uqlMaker)
        propertyList = []
        nodeProperty = res.alias('_nodeProperty').asProperties()
        edgeProeprty = res.alias('_edgeProperty').asProperties()
        propertyList.extend(nodeProperty)
        propertyList.extend(edgeProeprty)


        return propertyList
    def showNodeProperty(self, schemaName: str = None,
                         requestConfig: RequestConfig = RequestConfig()) -> List[Property]:
        '''
        Show all Node Schema properties.

        Args:
            schemaName: The name of schema

            requestConfig: An object of RequestConfig class

        Returns:
            List[Property]

        '''
        res = self.showProperty(schemaName=schemaName, dbType=DBType.DBNODE, requestConfig=requestConfig)
        return res

    def showEdgeProperty(self, schemaName: str = None,
                         requestConfig: RequestConfig = RequestConfig()) -> List[Property]:
        '''
        Show all Edge Schema properties.

        Args:
            schemaName: The name of schema

            requestConfig: An object of RequestConfig class

        Returns:
            List[Property]

        '''

        res = self.showProperty(schemaName=schemaName, dbType=DBType.DBEDGE, requestConfig=requestConfig)
        return res

    def createProperty(self, dbType: DBType, schemaName: str, prop: Property,
                       requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Create a property.

        Args:
            dbType: The DBType of data (DBNODE or DBEDGE)

            schema: The name of schema

            prop:  An object of Property class

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse

        '''
        command = dbType == DBType.DBNODE and CommandList.createNodeProperty or CommandList.createEdgeProperty
        commandP = ["@" + f"`{schemaName}`", f"`{prop.name}`",
                    getPropertyTypesDesc(prop.type, prop.subTypes)]

        if prop.description:
            commandP.append(prop.description)
        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        uqlMaker.setCommandParams(commandP=commandP)
        res = self.uqlSingle(uqlMaker)
        return res

    def dropProperty(self, dbType: DBType, schemaName: str, propertyName: str,
                     requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Drop a property.

        Args:
            dbType: The DBType of data (DBNODE or DBEDGE)

            schema: The name of schema

            property: The name of property

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse

        '''
        command = dbType == DBType.DBNODE and CommandList.dropNodeProperty or CommandList.dropEdgeProperty
        commandP = "@`%s`.`%s`" % (schemaName, propertyName)
        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        uqlMaker.setCommandParams(commandP=commandP)
        res = self.uqlSingle(uqlMaker)
        return res

    def alterProperty(self, dbType: DBType, property: Property, newProperty: Property,
                      requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Alter a property.

        Args:
            dbType: The DBType of data (DBNODE or DBEDGE)

            property: The name of property

            newProperty: The new name of property

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse

        '''
        command = dbType == DBType.DBNODE and CommandList.alterNodeProperty or CommandList.alterEdgeProperty
        commandP = "@`%s`.`%s`" % (property.schema, property.name)
        update_dict = {}
        if newProperty.name:
            update_dict.setdefault('name', newProperty.name)
        if newProperty.description:
            update_dict.update({'description': newProperty.description})
        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        uqlMaker.setCommandParams(commandP)
        uqlMaker.addParam("set", update_dict)
        res = self.uqlSingle(uqlMaker)
        return res



import time

import json
from typing import Tuple, List

from ultipa.configuration.RequestConfig import RequestConfig
from ultipa.operate.base_extra import BaseExtra
from ultipa.structs import GraphSet
from ultipa.types import ULTIPA, ULTIPA_RESPONSE
from ultipa.utils import UQLMAKER, CommandList
from ultipa.utils.ResposeFormat import ResponseKeyFormat
from ultipa.utils.convert import convertToGraph

REPLACE_KEYS = {
    "graph": "name",
}


class GraphExtra(BaseExtra):
    '''
    Processing class that defines settings for graphset related operations.
    '''

    def uqlCreateSubgraph(self, uql: str, subGraphName: str, requestConfig: RequestConfig = RequestConfig()):
        ret = self.uql(uql, requestConfig)
        graphRet = self.getGraph(subGraphName)
        if graphRet.status.code != 0:
            self.createGraph(GraphSet(name=subGraphName))
            time.sleep(3)

    def showGraph(self, requestConfig: RequestConfig = RequestConfig()) -> List[GraphSet]:
        '''
        Acquire graphset list.

        Args:
            requestConfig: An object of RequestConfig class

        Returns:
            List[GraphSet]
        '''

        uqlMaker = UQLMAKER(command=CommandList.showGraph, commonParams=requestConfig)
        uqlMaker.setCommandParams("")
        res = self.UqlListSimple(uqlMaker=uqlMaker, responseKeyFormat=ResponseKeyFormat(keyReplace=REPLACE_KEYS))
        res.data = convertToGraph(res)
        return res.data

    def getGraph(self, graphName: str,
                 requestConfig: RequestConfig = RequestConfig()) -> GraphSet:
        '''
        Acquire a designated graphset.

        Args:
            graphName: The name of graphset

            requestConfig: An object of RequestConfig class

        Returns:
            GraphSet
        '''

        uqlMaker = UQLMAKER(command=CommandList.showGraph, commonParams=requestConfig)
        uqlMaker.setCommandParams(graphName)

        res = self.UqlListSimple(uqlMaker=uqlMaker, responseKeyFormat=ResponseKeyFormat(keyReplace=REPLACE_KEYS))
        if res.status.code == ULTIPA.Code.SUCCESS:
            res.data = convertToGraph(res)
        else:
            res.data = None
        return res.data

    def createGraph(self, graph: GraphSet,
                    requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Create a graphset.

        Args:
            grpah: An object of Graph class

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse
        '''

        uqlMaker = UQLMAKER(command=CommandList.createGraph, commonParams=requestConfig)
        if graph.description:
            uqlMaker.setCommandParams([graph.name, graph.description])
        else:
            uqlMaker.setCommandParams(graph.name)
        res = self.uqlSingle(uqlMaker)
        return res

    def dropGraph(self, graphName: str,
                  requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Drop a graphset.

        Args:
            graphName: The name of graphset

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse
        '''

        uqlMaker = UQLMAKER(command=CommandList.dropGraph, commonParams=requestConfig)
        uqlMaker.setCommandParams(graphName)
        res = self.uqlSingle(uqlMaker)
        return res

    def alterGraph(self, oldGraph: GraphSet, newGraph: GraphSet,
                   requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Alter a graphset

        Args:
            oldGraphName: The orignal name of graphset

            newGraphName: The new name of graphset

            newDescription: The new description of graphset

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse
        '''

        requestConfig.graphName = oldGraph.name
        uqlMaker = UQLMAKER(command=CommandList.alterGraph, commonParams=requestConfig)
        uqlMaker.setCommandParams(oldGraph.name)
        data = {"name": newGraph.name}
        if newGraph.description is not None:
            data.update({'description': newGraph.description})
        uqlMaker.addParam("set", data)
        res = self.uqlSingle(uqlMaker)
        return res

    def hasGraph(self, graphName: str, requestConfig: RequestConfig = RequestConfig()) -> bool:
        '''
        Check if graph exists or not.

        Args:
            graphName: The name of graphset

            requestConfig: An object of RequestConfig class

        Returns:
            bool
        '''

        graphsdata = self.showGraph(requestConfig)
        for graphs in graphsdata:
            if (graphs.name == graphName):
                return True

        return False

    def createGraphIfNotExist(self, graph: GraphSet,
                              requestConfig: RequestConfig = RequestConfig()) -> Tuple[
        bool, ULTIPA_RESPONSE.UltipaResponse]:
        '''
        Checks if graph exists or not, if graph does not exist then creates new.

        Args:
            graphName: The name of graphset

            requestConfig: An object of RequestConfig class

        Returns:
            bool
            UltipaResponse
        '''

        if (self.hasGraph(graph.name, requestConfig)) == True:
            return [True, None]

        else:
            res = self.createGraph(graph, requestConfig)
            return [False, res]



# -*- coding: utf-8 -*-
# @Time    : 2024/05/14 11:47
# @Author  : Ultipa
# @Email   : support@ultipa.com
# @File    : Privilege.py
from typing import List
from ultipa.structs.BaseModel import BaseModel


class Process(BaseModel):
    '''
        Data class for Privilege.
    '''

    def __init__(self, processId: str, processUql: str, duration: str):
        self.processId = processId
        self.processUql = processUql
        self.duration = duration



from typing import List

from ultipa.types.types_response import User

from ultipa.operate.base_extra import BaseExtra
from ultipa.utils import UQLMAKER, CommandList
from ultipa.types import ULTIPA_REQUEST, ULTIPA_RESPONSE
from ultipa.utils.errors import ParameterException
from ultipa.utils.format import FormatType
from ultipa.utils.ResposeFormat import ResponseKeyFormat
from ultipa.configuration.RequestConfig import RequestConfig
from ultipa.proto import ultipa_pb2
from ultipa.connection.clientType import ClientType
from ultipa.types import ULTIPA, ULTIPA_REQUEST, ULTIPA_RESPONSE
from ultipa.utils.convert import convertToUser

JSONSTRING_KEYS = ["graphPrivileges", "systemPrivileges", "policies", "policy", "privilege"]
formatdata = ['graph_privileges']


class UserExtra(BaseExtra):
    '''
        Processing class that defines settings for user related operations.
    '''

    def _GRPATH_PRIVILEGES_DATA_FORMAT(self, obj):
        if isinstance(obj.get('graph_privileges'), list):
            resr = FormatType.graphPrivileges(obj.get('graph_privileges'))
            return resr
        else:
            return '[]'

    def showUser(self,
                 requestConfig: RequestConfig = RequestConfig()) -> List[User]:
        '''
        Show user list.

        Args:
            requestConfig: An object of RequestConfig class

        Returns:
            List[User]

        '''

        uqlMaker = UQLMAKER(command=CommandList.showUser, commonParams=requestConfig)
        res = self.UqlListSimple(uqlMaker=uqlMaker,
                                 responseKeyFormat=ResponseKeyFormat(jsonKeys=JSONSTRING_KEYS, dataFormat=formatdata))
        if len(res.data) > 0:
            res.data = convertToUser(res)
        else:
            res.data = None
        return res.data

    def getUser(self, userName: str,
                requestConfig: RequestConfig = RequestConfig()) -> User:
        '''
        Get a designated user.

        Args:
            username: The name of user

            requestConfig: An object of RequestConfig class

        Returns:
            User

        '''

        uqlMaker = UQLMAKER(command=CommandList.getUser, commonParams=requestConfig)
        uqlMaker.setCommandParams(userName)
        res = self.UqlListSimple(uqlMaker=uqlMaker,
                                 responseKeyFormat=ResponseKeyFormat(jsonKeys=JSONSTRING_KEYS, dataFormat=formatdata))
        # if isinstance(res.data, list) and len(res.data) > 0:
        # 	res.data = res.data[0]
        if res.status.code == ULTIPA.Code.SUCCESS:
            res.data = convertToUser(res)
        return res.data

    def getSelfInfo(self,
                    requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Get the current user.

        Args:
            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse

        '''
        uqlMaker = UQLMAKER(command=CommandList.getSelfInfo, commonParams=requestConfig)
        res = self.UqlListSimple(uqlMaker=uqlMaker,
                                 responseKeyFormat=ResponseKeyFormat(jsonKeys=JSONSTRING_KEYS, dataFormat=formatdata))

        return res

    def createUser(self, request: ULTIPA_REQUEST.CreateUser,
                   requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Create a user.

        Args:
            request: An object of CreateUser class

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse

        '''

        uqlMaker = UQLMAKER(command=CommandList.createUser, commonParams=requestConfig)
        params = []
        if request.username:
            params.append(request.username)
        else:
            raise ParameterException(err='username is a required parameter')

        if request.password:
            params.append(request.password)
        else:
            raise ParameterException(err='password is a required parameter')

        if request.graph_privileges:
            params.append(request.graph_privileges)
        else:
            params.append({})

        if request.system_privileges:
            params.append(request.system_privileges)
        else:
            params.append([])

        if request.policies:
            params.append(request.policies)
        else:
            params.append([])

        if request.property_privileges:
            params.append(request.property_privileges)
        else:
            params.append({})

        uqlMaker.setCommandParams(params)
        return self.uqlSingle(uqlMaker=uqlMaker)

    def dropUser(self, userName: str,
                 requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Drop a user.

        Args:
            userName: The name of user

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse

        '''

        uqlMaker = UQLMAKER(command=CommandList.dropUser, commonParams=requestConfig)
        uqlMaker.setCommandParams(userName)
        return self.uqlSingle(uqlMaker=uqlMaker)

    def alterUser(self, request: ULTIPA_REQUEST.AlterUser,
                  requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Alter a user.

        Args:
            request: An object of AlterUser class

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse

        '''

        uqlMaker = UQLMAKER(command=CommandList.alterUser, commonParams=requestConfig)
        if request.username:
            uqlMaker.setCommandParams(request.username)
        else:
            raise ParameterException(err='username is a required parameter')

        paramsDict = {}
        if request.password:
            paramsDict.setdefault('password', request.password)

        if request.graph_privileges:
            paramsDict.setdefault('graph_privileges', request.graph_privileges)

        if request.system_privileges:
            paramsDict.setdefault('system_privileges', request.system_privileges)

        if request.policies:
            paramsDict.setdefault('policies', request.policies)

        if request.property_privileges:
            paramsDict.setdefault('property_privileges', request.property_privileges)

        uqlMaker.addParam('set', paramsDict)
        return self.uqlSingle(uqlMaker=uqlMaker)

    def getUserSetting(self, request: ULTIPA_REQUEST.getUserSetting,
                       requestconfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:

        '''
        Get User Settings .

        Args:
            request: An object of getUserSetting Class

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse

        '''

        try:
            request = ultipa_pb2.UserSettingRequest(user_name=request.username, type=request.type,
                                                    opt=ultipa_pb2.UserSettingRequest.OPTION.OPT_GET)
            clientInfo = self.getClientInfo(graphSetName=requestconfig.graphName, useMaster=requestconfig.useMaster,
                                            clientType=ClientType.Leader,
                                            isGlobal=True, timezone=requestconfig.timeZone,
                                            timeZoneOffset=requestconfig.timeZoneOffset)
            res = clientInfo.Controlsclient.UserSetting(request, metadata=clientInfo.metadata)
            status = FormatType.status(res.status)
            data = res.data
            response = ULTIPA_RESPONSE.Response(status=status, data=data)
            return response

        except Exception as e:
            errorRes = ULTIPA_RESPONSE.Response()
            try:
                message = str(e._state.code) + ' : ' + str(e._state.details)
            except:
                message = 'UNKNOWN ERROR'
            errorRes.status = ULTIPA.Status(code=ULTIPA.Code.UNKNOW_ERROR, message=message)
            return errorRes

    def setUserSetting(self, request: ULTIPA_REQUEST.setUserSetting,
                       requestconfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:

        '''
        set User Settings .

        Args:
            request: An object of setUserSetting Class

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse

        '''
        try:
            request = ultipa_pb2.UserSettingRequest(user_name=request.username, type=request.type,
                                                    opt=ultipa_pb2.UserSettingRequest.OPTION.OPT_SET, data=request.data)
            clientInfo = self.getClientInfo(graphSetName=requestconfig.graphName, useMaster=requestconfig.useMaster,
                                            clientType=ClientType.Leader, isGlobal=True,
                                            timezone=requestconfig.timeZone,
                                            timeZoneOffset=requestconfig.timeZoneOffset)
            res = clientInfo.Controlsclient.UserSetting(request, metadata=clientInfo.metadata)
            status = FormatType.status(res.status)
            response = ULTIPA_RESPONSE.Response(status=status)
            return response



        except Exception as e:
            errorRes = ULTIPA_RESPONSE.Response()
            try:
                message = str(e._state.code) + ' : ' + str(e._state.details)
            except:
                message = 'UNKNOWN ERROR'
            errorRes.status = ULTIPA.Status(code=ULTIPA.Code.UNKNOW_ERROR, message=message)
            return errorRes



from ultipa.configuration.RequestConfig import RequestConfig
from ultipa.operate.base_extra import BaseExtra
from ultipa.structs import Schema
from ultipa.structs import DBType
from ultipa.types import ULTIPA, ULTIPA_REQUEST, ULTIPA_RESPONSE
from ultipa.utils import UQLMAKER, CommandList
from ultipa.utils.ResposeFormat import ResponseKeyFormat
from typing import Tuple, List
from ultipa.operate.property_extra import PropertyExtra
from ultipa.structs.Property import Property

BOOL_KEYS = ["index", "lte"]
REPLACE_KEYS = {
    "name": "schemaName",
    "type": "propertyType",
}


class SchemaExtra(BaseExtra):
    '''
        Prcessing class that defines settings for schema related operations.
    '''

    def createSchema(self, schema: Schema, isCreateProperties: bool = False,
                     requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Create a schema.

        Args:
            schema: An object of Schema class

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse

        '''

        command = schema.DBType == DBType.DBNODE and CommandList.createNodeSchema or CommandList.createEdgeSchema
        commandP = [f"`{schema.name}`"]
        if schema.description:
            commandP.append(schema.description)
        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        uqlMaker.setCommandParams(commandP=commandP)
        res = self.uqlSingle(uqlMaker=uqlMaker)

        if isCreateProperties:
            if schema.properties:
                for prop in schema.properties:
                    try:
                        res1 = PropertyExtra.createProperty(self, dbType=schema.DBType, schemaName=schema.name,
                                                            prop=prop, requestConfig=requestConfig)
                    except Exception as e:
                        print("An error occurred while creating property:", prop.name, "Error:", e)
        return res

    def showSchema(self,
                   requestConfig: RequestConfig = RequestConfig()) -> List[Schema]:
        '''
        Show schema(s).

        Args:
            requestConfig: An object of RequestConfig class

        Returns:
            List[Schema]

        '''

        command = CommandList.showSchema
        commandP = ''

        uqlMaker = UQLMAKER(command=command, commandP=commandP, commonParams=requestConfig)
        res = self.uqlSingle(uqlMaker)
        schemalist = []
        nodeschemas = res.alias("_nodeSchema").asSchemas()
        edgeschemas = res.alias("_edgeSchema").asSchemas()
        schemalist.extend(nodeschemas)
        schemalist.extend(edgeschemas)
        return schemalist

    def showNodeSchema(self, requestConfig: RequestConfig = RequestConfig()) -> List[Schema]:
        '''
        Show Nodeschema(s).

        Args:
            requestConfig: An object of RequestConfig class

        Returns:
            List[Schema]

        '''

        command = CommandList.showNodeSchema
        commandp = ''
        # 	if schemaName:
        # 		commandP = '@' + schemaName
        # 	else:
        # 		commandP = ''
        # else:
        # 	command = CommandList.showSchema
        # 	commandP = ''
        uqlMaker = UQLMAKER(command=command, commandP=commandp, commonParams=requestConfig)
        res = self.uqlSingle(uqlMaker)
        res.items = res.alias("_nodeSchema").asSchemas()
        return res.items

    def showEdgeSchema(self, requestConfig: RequestConfig = RequestConfig()) -> List[Schema]:
        '''
        Show Edgeschema(s).

        Args:
            requestConfig: An object of RequestConfig class

        Returns:
            List[Schema]

        '''

        command = CommandList.showEdgeSchema
        commandp = ''
        # 	if schemaName:
        # 		commandP = '@' + schemaName
        # 	else:
        # 		commandP = ''
        # else:
        # 	command = CommandList.showSchema
        # 	commandP = ''

        # uqlMaker = UQLMAKER(command=command, commandP=commandP, commonParams=requestConfig)
        uqlMaker = UQLMAKER(command=command, commandP=commandp, commonParams=requestConfig)
        res = self.uqlSingle(uqlMaker)
        res.items = res.alias("_edgeSchema").asSchemas()
        return res.items

    def alterSchema(self, schema: Schema, newSchema: Schema,
                    requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Alter schema.

        Args:
            schema: An object of schema class(to be altered)
            newSchema: An object of schema class

        Returns:
             UltipaResponse

        '''
        command = schema.DBType == DBType.DBNODE and CommandList.alterNodeSchema or CommandList.alterEdgeSchema
        # commandP = '@' + schema.name
        commandP = "@`%s`" % (schema.name)
        update_dict = {}
        if newSchema.name:
            update_dict.setdefault('name', newSchema.name)
        if newSchema.description:
            update_dict.update({'description': newSchema.description})
        uqlMaker = UQLMAKER(command=command, commandP=commandP, commonParams=requestConfig)
        uqlMaker.addParam("set", update_dict)
        res = self.uqlSingle(uqlMaker=uqlMaker)
        return res

    def dropSchema(self, schema: Schema,
                   requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Drop schema.

        Args:
            schema: An object of schema class

        Returns:
             UltipaResponse

        '''

        command = schema.DBType == DBType.DBNODE and CommandList.dropNodeSchema or CommandList.dropEdgeSchema
        # commandP = '@' + schema.name
        commandP = "@`%s`" % (schema.name)

        uqlMaker = UQLMAKER(command=command, commandP=commandP, commonParams=requestConfig)
        res = self.uqlSingle(uqlMaker=uqlMaker)
        return res

    def getSchema(self, schemaName: str, dbType: DBType,
                  requestConfig: RequestConfig = RequestConfig()) -> Schema:
        '''
        Acquire a designated Schema.

        Args:
            schemaName: The name of Schema

            dbType:The DBType of data (DBNODE or DBEDGE)

            requestConfig: An object of RequestConfig class

        Returns:
            Schema
        '''

        if dbType != None:
            if dbType == DBType.DBNODE:
                command = CommandList.showNodeSchema
            elif dbType == DBType.DBEDGE:
                command = CommandList.showEdgeSchema

            if schemaName:
                commandP = "@`%s`" % (schemaName)

        uqlMaker = UQLMAKER(command, commandP=commandP, commonParams=requestConfig)

        res = self.uqlSingle(uqlMaker=uqlMaker)
        if res.status.code == ULTIPA.Code.SUCCESS:
            if dbType == DBType.DBNODE:
                res.items = res.alias("_nodeSchema").asSchemas()[0]
            else:
                res.items = res.alias("_edgeSchema").asSchemas()[0]
        return res.items

    def getNodeSchema(self, schemaName: str,
                      requestConfig: RequestConfig = RequestConfig()) -> Schema:
        '''
        Acquire a designated Node Schema.

        Args:
            schemaName: The name of Schema
            requestConfig: An object of RequestConfig class

        Returns:
            Schema
        '''

        command = CommandList.showNodeSchema

        if schemaName:
            commandP = "@`%s`" % (schemaName)

        uqlMaker = UQLMAKER(command, commandP=commandP, commonParams=requestConfig)

        res = self.uqlSingle(uqlMaker=uqlMaker)
        if res.status.code == ULTIPA.Code.SUCCESS:
            res.items = res.alias("_nodeSchema").asSchemas()[0]
        else:
            res.items = None
        return res.items

    def getEdgeSchema(self, schemaName: str,
                      requestConfig: RequestConfig = RequestConfig()) -> Schema:
        '''
        Acquire a designated Edge Schema.

        Args:
            schemaName: The name of Schema
            requestConfig: An object of RequestConfig class

        Returns:
            ResponseSchema
        '''

        command = CommandList.showEdgeSchema

        if schemaName:
            commandP = "@`%s`" % (schemaName)

        uqlMaker = UQLMAKER(command, commandP=commandP, commonParams=requestConfig)

        res = self.uqlSingle(uqlMaker=uqlMaker)
        if res.status.code == ULTIPA.Code.SUCCESS:
            res.items = res.alias("_edgeSchema").asSchemas()[0]
        else:
            res.items = None
        return res.items

    def createSchemaIfNotExist(self, schema: Schema, isCreateProperties: bool = False,
                               requestConfig: RequestConfig = RequestConfig()) -> Tuple[
        bool, ULTIPA_RESPONSE.UltipaResponse]:
        '''
        Create a schema if schema does not exist.

        Args:
            schema: An object of Schema class

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse

        '''
        check = self.getSchema(schemaName=schema.name, dbType=schema.DBType, requestConfig=requestConfig)
        if check is not None:
            return [True, None]

        elif check is None:
            res = self.createSchema(schema=schema, isCreateProperties=isCreateProperties, requestConfig=requestConfig)
            return [False, res]

    def getProperty(self, dbType: DBType, propertyName: str, schemaName: str,
                    requestConfig: RequestConfig = RequestConfig()) -> Property:
        '''
        Get a property.

        Args:
            dbType: The DBType of data (DBNODE or DBEDGE)

            schemaName: The name of schema

            propertyName: The name of the Property

            requestConfig: An object of RequestConfig class

        Returns:
            Property

        '''
        res = self.getSchema(schemaName=schemaName, dbType=dbType, requestConfig=requestConfig)
        properties = res.properties
        for property in properties:
            if property.name == propertyName:
                return property
        return None

    def getNodeProperty(self, propertyName: str, schemaName: str,
                        requestConfig: RequestConfig = RequestConfig()) -> Property:
        '''
        Get a Node property.

        Args:
            schemaName: The name of schema

            propertyName: The name of the Property

            requestConfig: An object of RequestConfig class

        Returns:
            Property

        '''
        res = self.getProperty(propertyName=propertyName, schemaName=schemaName, dbType=DBType.DBNODE,
                               requestConfig=requestConfig)
        return res

    def getEdgeProperty(self, propertyName: str, schemaName: str,
                        requestConfig: RequestConfig = RequestConfig()) -> Property:
        '''
        Get an Edge property.

        Args:
            schemaName: The name of schema

            propertyName: The name of the Property

            requestConfig: An object of RequestConfig class

        Returns:
            Property

        '''
        res = self.getProperty(propertyName=propertyName, schemaName=schemaName, dbType=DBType.DBEDGE,
                               requestConfig=requestConfig)
        return res

    def createPropertyIfNotExist(self, dbType: DBType, schemaName: str, prop: Property,
                                 requestConfig: RequestConfig = RequestConfig()) -> Tuple[
        bool, ULTIPA_RESPONSE.ResponseProperty]:
        '''
        Create a schema if schema does not exist.

        Args:
            schema: An object of Schema class

            requestConfig: An object of RequestConfig class

        Returns:
            Bool
            ResponseGetProperty

        '''

        res = self.getProperty(dbType=dbType, propertyName=prop.name, schemaName=schemaName,
                               requestConfig=requestConfig)
        if res != None:
            return [True, None]
        if res == None:
            newprop = PropertyExtra.createProperty(self, dbType=dbType, schemaName=schemaName, prop=prop,requestConfig=requestConfig)
            return [False, newprop]



from ultipa.types.types_response import Task, Top

from ultipa.configuration.RequestConfig import RequestConfig
from ultipa.operate.base_extra import BaseExtra
from ultipa.types import ULTIPA_REQUEST, ULTIPA, ULTIPA_RESPONSE
from ultipa.utils import UQLMAKER, CommandList
import json
from typing import Union, Optional, List
from ultipa.utils.convert import convertToAnyObject, convertToTask
from ultipa.utils.ResposeFormat import ResponseKeyFormat
from ultipa.types.types_request import TaskStatus
from ultipa.utils.convert import convertToTop


class ALGO_RETURN_TYPE:
    ALGO_RETURN_REALTIME = 1
    ALGO_RETURN_WRITE_BACK = 2
    ALGO_RETURN_VISUALIZATION = 4


class TaskExtra(BaseExtra):
    '''
    Processing class that defines settings for task and process related operations.
    '''

    def top(self,
            requestConfig: RequestConfig = RequestConfig()) -> List[Top]:
        '''
        Top real-time processes.

        Args:
            requestConfig: An object of RequestConfig class

        Returns:
            List[Top]

        '''
        uqlMaker = UQLMAKER(command=CommandList.top, commonParams=requestConfig)
        res = self.UqlListSimple(uqlMaker)
        if len(res.data) > 0:
            res.data = convertToTop(res)
        else:
            res.data = None
        return res.data

    def kill(self, processId: str,
             requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Kill real-time processes.

        Args:
            id: The ID of real-time process

            all: Whether to kill all real-time processes

            requestConfig: An object of RequestConfig class

        Returns:
            Response

        '''

        commonP = processId
        uqlMaker = UQLMAKER(command=CommandList.kill, commonParams=requestConfig)
        uqlMaker.setCommandParams(commonP)
        res = self.uqlSingle(uqlMaker)
        return res

    def showTask(self, algoNameOrId: Optional[Union[int, str]] = None, status: TaskStatus = None,
                 config: RequestConfig = RequestConfig()) -> List[Task]:
        '''
        Show back-end tasks.

        Args:

            algoNameOrld (str): The name of the task algo to show

            status (str): The status of the tasks to retrieve.

            config (RequestConfig): An object of RequestConfig class.

        Returns:
            List[Task]

        '''

        _jsonKeys = ['taskJson']
        uqlMaker = UQLMAKER(command=CommandList.showTask, commonParams=config)
        commonP = []

        # if all:
        # 	commonP.append('*')

        # else:

        if isinstance(algoNameOrId, str) or algoNameOrId is None:
            if algoNameOrId and status:
                commonP.append(algoNameOrId)
                commonP.append(status.value)
            if algoNameOrId and not status:
                commonP.append(algoNameOrId)
                commonP.append('*')
            if not algoNameOrId and status:
                commonP.append('*')
                commonP.append(status.value)

        elif isinstance(algoNameOrId, int):
            commonP = algoNameOrId

        uqlMaker.setCommandParams(commandP=commonP)
        res = self.UqlListSimple(uqlMaker=uqlMaker, responseKeyFormat=ResponseKeyFormat(jsonKeys=_jsonKeys))
        newDatas = []
        if res.data:
            for obj in res.data:
                obj = obj.__dict__
                newData = ULTIPA_RESPONSE.Task()
                taskJson = obj.get("taskJson", {})
                newData.param = json.loads(taskJson.get("param", "{}"))
                newData.result = taskJson.get("result")
                task_info = taskJson.get("task_info", {})

                if task_info.get('status_code'):
                    task_info["status_code"] = ULTIPA.TaskStatusString[task_info.get("TASK_STATUS")]
                if task_info.get('engine_cost'):
                    task_info["engine_cost"] = task_info.get("writing_start_time", 0) - task_info.get("start_time", 0)
                newData.task_info = convertToTask(task_info)

                return_type_get = int(task_info.get('return_type', 0))
                return_type = ULTIPA_RESPONSE.Return_Type()
                return_type.is_realtime = True if return_type_get & ALGO_RETURN_TYPE.ALGO_RETURN_REALTIME else False
                return_type.is_visualization = True if return_type_get & ALGO_RETURN_TYPE.ALGO_RETURN_VISUALIZATION else False
                return_type.is_wirte_back = True if return_type_get & ALGO_RETURN_TYPE.ALGO_RETURN_WRITE_BACK else False
                newData.task_info.__setattr__('return_type', return_type)
                newDatas.append(newData)
            res.data = newDatas
        else:
            res.data = None
        return res.data

    def clearTask(self, algoNameOrId: Optional[Union[int, str]] = None, status: TaskStatus = None,
                  config: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Clear back-end tasks.

        Args:

            algoNameOrId (str): The name of the algotask to clear.

            status (str): The status of the tasks to clear.

            requestConfig (RequestConfig): An object of RequestConfig class.

        Returns:
            Response

        '''

        uqlMaker = UQLMAKER(command=CommandList.clearTask, commonParams=config)
        commonP = []
        # if all:
        # 	commonP.append('*')
        # else:

        if isinstance(algoNameOrId, str) or algoNameOrId is None:
            if algoNameOrId and status:
                commonP.append(algoNameOrId)
                commonP.append(status.value)
            if algoNameOrId and not status:
                commonP.append(algoNameOrId)
                commonP.append('*')
            if not algoNameOrId and status:
                commonP.append('*')
                commonP.append(status.value)

        elif isinstance(algoNameOrId, int):
            commonP = algoNameOrId
        # if id:
        # 		commonP = id

        uqlMaker.setCommandParams(commandP=commonP)
        res = self.uqlSingle(uqlMaker)

        return res

    def stopTask(self, id: str = None,
                 config: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Stop back-end tasks.

        Args:
            id: The ID of back-end task

            all: Whether to stop all back-end tasks that are computing

            requestConfig: An object of RequestConfig class

        Returns:
            Response

        '''
        uqlMaker = UQLMAKER(command=CommandList.stopTask, commonParams=config)
        commonP = []
        # if all:
        # 	commonP = '*'
        # if id:
        commonP = id
        uqlMaker.setCommandParams(commandP=commonP)
        return self.uqlSingle(uqlMaker)

    def pauseTask(self, id: int = None, all: bool = False,
                  requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.ResponseCommon:
        '''
        Stop back-end tasks.

        Args:
            id: The ID of back-end task

            all: Whether to pause all back-end tasks that are computing

            requestConfig: An object of RequestConfig class

        Returns:
            ResponseCommon

        '''
        uqlMaker = UQLMAKER(command=CommandList.pauseTask, commonParams=requestConfig)
        commonP = []
        if all:
            commonP = '*'
        if id:
            commonP = id
        uqlMaker.setCommandParams(commandP=commonP)
        return self.UqlUpdateSimple(uqlMaker)

    def resumeTask(self, id: int = None, all: bool = False,
                   requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.ResponseCommon:
        '''
        Stop back-end tasks.

        Args:
            id: The ID of back-end task

            all: Whether to resume all back-end tasks

            requestConfig: An object of RequestConfig class

        Returns:
            ResponseCommon

        '''
        uqlMaker = UQLMAKER(command=CommandList.resumeTask, commonParams=requestConfig)
        commonP = []
        if all:
            commonP = '*'
        if id:
            commonP = id
        uqlMaker.setCommandParams(commandP=commonP)
        return self.UqlUpdateSimple(uqlMaker)

    def clusterInfo(self,
                    requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.ClusterInfo:
        '''
        Show cluster information.

        Args:
            requestConfig: An object of RequestConfig class

        Returns:
            ClusterInfo

        '''
        self.refreshRaftLeader(redirectHost='', requestConfig=requestConfig)
        result = []
        if not requestConfig.graphName:
            graphSetName = 'default'
        else:
            graphSetName = requestConfig.graphName
        for peer in self.hostManagerControl.getAllHostStatusInfo(graphSetName):
            info = ULTIPA_RESPONSE.Cluster()
            info.status = peer.status
            info.host = peer.host
            info.isLeader = peer.isLeader
            info.isFollowerReadable = peer.isFollowerReadable
            info.isAlgoExecutable = peer.isAlgoExecutable
            info.isUnset = peer.isUnset
            info.cpuUsage = None
            info.memUsage = None
            if peer.status:
                ret = self.stats(requestConfig=RequestConfig(host=peer.host))
                if ret.status.code == ULTIPA.Code.SUCCESS:
                    info.cpuUsage = ret.data.cpuUsage
                    info.memUsage = ret.data.memUsage

            result.append(info)
        res = ULTIPA_RESPONSE.Response()
        res.data = result
        return res



from typing import List

from ultipa import DBType
from ultipa.operate.base_extra import BaseExtra
from ultipa.types import ULTIPA_REQUEST, ULTIPA, ULTIPA_RESPONSE
from ultipa.structs.Property import Property
from ultipa.structs.Index import Index
from ultipa.utils import UQLMAKER, CommandList
from ultipa.configuration.RequestConfig import RequestConfig
from ultipa.structs.DBType import DBType
from ultipa.utils.convert import convertToIndex
from ultipa.utils.convert import convertToFullText


class IndexExtra(BaseExtra):
    '''
    Processing class that defines settings for index related operations.
    '''

    def showIndex(self,
                  requestConfig: RequestConfig = RequestConfig()) -> List[Index]:
        '''
        Show all indice.

        Args:
            requestConfig: An object of RequestConfig class

        Returns:
            List[Index]
        '''
        # if dbType != None:
        # 	command = dbType == DBType.DBNODE and CommandList.showNodeIndex or CommandList.showEdgeIndex
        # else:
        command = CommandList.showIndex
        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        res = self.UqlListSimple(uqlMaker=uqlMaker, isSingleOne=False)
        indexdata = convertToIndex(res=res, all=True)
        if len(indexdata) > 0:
            res.data = indexdata
        else:
            res.data = None
        return res.data

    def showNodeIndex(self, requestConfig: RequestConfig = RequestConfig()) -> List[Index]:
        '''
        Show all Node indice.

        Args:
            requestConfig: An object of RequestConfig class

        Returns:
            List[Index]
        '''
        # res=self.showIndex(dbType=DBType.DBNODE,requestConfig=requestConfig)
        command = CommandList.showNodeIndex
        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        res = self.UqlListSimple(uqlMaker=uqlMaker, isSingleOne=False)
        indexdata = convertToIndex(res=res, all=False, dbtype=DBType.DBNODE)
        if len(indexdata) > 0:
            res.data = indexdata
        else:
            res.data = None
        return res.data

    def showEdgeIndex(self, requestConfig: RequestConfig = RequestConfig()) -> List[Index]:
        '''
        Show all Edge indice.

        Args:
            requestConfig: An object of RequestConfig class

        Returns:
            List[Index]
        '''
        # res=self.showIndex(dbType=DBType.DBEDGE,requestConfig=requestConfig)
        command = CommandList.showEdgeIndex
        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        res = self.UqlListSimple(uqlMaker=uqlMaker, isSingleOne=False)
        indexdata = convertToIndex(res=res, all=False, dbtype=DBType.DBEDGE)
        if len(indexdata) > 0:
            res.data = indexdata
        else:
            res.data = None
        return res.data

    def showFulltext(self,
                     requestConfig: RequestConfig = RequestConfig()) -> List[Index]:
        '''
        Show all full-text indice.

        Args:
            requestConfig: An object of RequestConfig class
        Returns:

            List[Index]
        '''
        # if dbType != None:
        # 	command = dbType == DBType.DBNODE and CommandList.showNodeFulltext or CommandList.showEdgeFulltext
        # else:
        command = CommandList.showFulltext
        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        res = self.UqlListSimple(uqlMaker=uqlMaker, isSingleOne=False)
        fulltextdata = convertToFullText(res=res)
        if len(fulltextdata) > 0:
            res.data = fulltextdata
        else:
            res.data = None
        return res.data

    def showNodeFulltext(self, requestConfig: RequestConfig = RequestConfig()) -> List[Index]:
        '''
        Show all full-text Node indice.

        Args:

            requestConfig: An object of RequestConfig class

        Returns:

            List[Index]
        '''
        # res=self.showFulltext(dbType=DBType.DBNODE,requestConfig=requestConfig)
        command = CommandList.showNodeFulltext
        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        res = self.UqlListSimple(uqlMaker=uqlMaker, isSingleOne=False)
        fulltextdata = convertToFullText(res=res, all=False, dbtype=DBType.DBNODE)
        if len(fulltextdata) > 0:
            res.data = fulltextdata
        else:
            res.data = None
        return res.data

    def showEdgeFulltext(self, requestConfig: RequestConfig = RequestConfig()) -> List[Index]:
        '''
        Show all full-text Edge indice.

        Args:

            requestConfig: An object of RequestConfig class

        Returns:

            List[Index]
        '''
        # res=self.showFulltext(dbType=DBType.DBEDGE,requestConfig=requestConfig)
        command = CommandList.showEdgeFulltext
        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        res = self.UqlListSimple(uqlMaker=uqlMaker, isSingleOne=False)
        fulltextdata = convertToFullText(res=res, all=False, dbtype=DBType.DBEDGE)
        if len(fulltextdata) > 0:
            res.data = fulltextdata
        else:
            res.data = None
        return res.data

    def createIndex(self, dbType: DBType, propertyName: str, schemaName: str = None,
                    requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Create an index.

        Args:
            dbType: The DBType of data (DBNODE or DBEDGE)

            schemaName: The name of schema

            property: An object of Property class

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse
        '''

        command = dbType == DBType.DBNODE and CommandList.createNodeIndex or CommandList.createEdgeIndex
        # commandP = request.toString
        if schemaName:
            commandP = "@`%s`.`%s`" % (schemaName, propertyName)
        elif schemaName == None:
            commandP = "@`*`.`%s`" % (propertyName)
        uqlMaker = UQLMAKER(command=command, commandP=commandP, commonParams=requestConfig)
        res = self.uqlSingle(uqlMaker=uqlMaker)
        return res

    def createFulltext(self, dbType: DBType, schemaName: str, propertyName: str, fulltextName: str,
                       requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Create a full-text index.

        Args:
            dbType: The DBType of data (DBNODE or DBEDGE)

            schemaName: The name of schema

            property: An object of Property class

            fulltextName: Name of the fulltext index

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse

        '''
        command = dbType == DBType.DBNODE and CommandList.createNodeFulltext or CommandList.createEdgeFulltext
        command1 = "@`%s`.`%s`" % (schemaName, propertyName)
        commandP = [command1, fulltextName]
        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        uqlMaker.setCommandParams(commandP=commandP)
        res = self.uqlSingle(uqlMaker=uqlMaker)
        return res

    def dropIndex(self, dbType: DBType, propertyName: str, schemaName: str = None,
                  requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Drop an index.

        Args:
            dbType: The DBType of data (DBNODE or DBEDGE)

            schemaName: The name of schema

            propertyName: The name of property

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse
        '''
        command = dbType == DBType.DBNODE and CommandList.dropNodeIndex or CommandList.dropEdgeIndex
        # commandP = request.toString
        if schemaName:
            commandP = "@`%s`.`%s`" % (schemaName, propertyName)
        elif schemaName == None:
            commandP = "@`*`.`%s`" % (propertyName)

        uqlMaker = UQLMAKER(command=command, commandP=commandP, commonParams=requestConfig)
        res = self.uqlSingle(uqlMaker=uqlMaker)
        return res

    def dropFulltext(self, dbType: DBType, fulltextName: str,
                     requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Drop a full-text index.

        Args:
            dbType: The DBType of data (DBNODE or DBEDGE)

            fulltextName: Name of the fulltext index

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse
        '''

        command = dbType == DBType.DBNODE and CommandList.dropNodeFulltext or CommandList.dropEdgeFulltext
        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        uqlMaker.setCommandParams(fulltextName)
        res = self.uqlSingle(uqlMaker=uqlMaker)
        return res
